#ifndef __BEAM_H
#define __BEAM_H

#include <vector>
#include "AxisAlignedBox3.h"

using namespace std;

class KdNode;
class Intersectable;

class Beam {
  
public:			
  enum {STORE_KD_NODES=1, STORE_OBJECTS=2, VALID=4};
  int mFlags;
  
  // list of nodes intersected by the frustum
  vector<KdNode *> mKdNodes;
  // list of objects intersected by the frustum
  ObjectContainer mObjects;
  // view cells intersected by frustum
  ViewCellContainer mViewCells;

  // spatial box
  AxisAlignedBox3 mBox;
  // directional box (it is actually a 2D box - only x and y ranges are valid)
  // directional parametrization according to VssRay::GetDirParametrization
  AxisAlignedBox3 mDirBox;

  vector<Plane3> mPlanes;

    
  void Construct(const AxisAlignedBox3 &box,
	  const AxisAlignedBox3 &dBox);
  
  void ComputeFrustum(float &left, float &right,
					  float &bottom, float &top,
					  float &near, float &far,
					  const AxisAlignedBox3 &sceneBBox) const;

  int ComputeIntersection(const AxisAlignedBox3 &box);

  Vector3 GetMainDirection() const;

  bool IsValid() { return mFlags & VALID; }
  bool SetValid() { return mFlags |= VALID; }

  Beam():mFlags(STORE_KD_NODES+STORE_OBJECTS), mKdNodes(0), mObjects(0)
  {
  }
};


#endif
